@@ -13,8 +13,11 @@ sudo installer -pkg ./Packages.pkg -target /
 
 brew update
 
-#Base OBS Deps
-brew install qt5 jack
+#Base OBS Deps and ccache
+brew install qt5 jack speexdsp ccache
+
+export PATH=/usr/local/opt/ccache/libexec:$PATH
+ccache -s || echo "CCache is not available."
 
 # Fetch and untar prebuilt OBS deps that are compatible with older versions of OSX
 curl -L -O https://s3-us-west-2.amazonaws.com/obs-nightly/osx-deps.tar.gz -f --retry 5 -C -
@@ -28,11 +31,12 @@ unzip -q ./vlc-master.zip
 curl -L -o ./sparkle.tar.bz2 https://github.com/sparkle-project/Sparkle/releases/download/1.16.0/Sparkle-1.16.0.tar.bz2
 mkdir ./sparkle
 tar -xf ./sparkle.tar.bz2 -C ./sparkle
+sudo cp -R ./sparkle/Sparkle.framework /Library/Frameworks/Sparkle.framework
 
 # CEF Stuff
-curl -kLO http://opensource.spotify.com/cefbuilds/cef_binary_3.2883.1540.gedbfb20_macosx64.tar.bz2 -f --retry 5 -C -
-tar -xf ./cef_binary_3.2883.1540.gedbfb20_macosx64.tar.bz2
-cd ./cef_binary_3.2883.1540.gedbfb20_macosx64
+curl -kLO https://obs-nightly.s3-us-west-2.amazonaws.com/cef_binary_${CEF_BUILD_VERSION}_macosx64.tar.bz2 -f --retry 5 -C -
+tar -xf ./cef_binary_${CEF_BUILD_VERSION}_macosx64.tar.bz2
+cd ./cef_binary_${CEF_BUILD_VERSION}_macosx64
 mkdir build
 cd ./build
 cmake -DCMAKE_CXX_FLAGS="-std=c++11 -stdlib=libc++" -DCMAKE_EXE_LINKER_FLAGS="-std=c++11 -stdlib=libc++" -DCMAKE_OSX_DEPLOYMENT_TARGET=10.9 ..
