@@ -119,6 +119,10 @@ private:
 	bool projectChanged = false;
 	bool previewEnabled = true;
 
+	const char *copyString;
+	const char *copyFiltersString;
+	bool copyVisible = true;
+
 	QPointer<QThread> updateCheckThread;
 	QPointer<QThread> logUploadThread;
 
@@ -413,6 +417,13 @@ private slots:
 
 	void ToggleShowHide();
 
+	void on_actionCopySource_triggered();
+	void on_actionPasteRef_triggered();
+	void on_actionPasteDup_triggered();
+
+	void on_actionCopyFilters_triggered();
+	void on_actionPasteFilters_triggered();
+
 private:
 	/* OBS Callbacks */
 	static void SceneReordered(void *data, calldata_t *params);
