@@ -279,15 +279,15 @@ EXPORT const char *obs_get_locale(void);
 EXPORT profiler_name_store_t *obs_get_profiler_name_store(void);
 
 /**
- * Sets base video ouput base resolution/fps/format.
+ * Sets base video output base resolution/fps/format.
  *
- * @note This data cannot be changed if an output is corrently active.
+ * @note This data cannot be changed if an output is currently active.
  * @note The graphics module cannot be changed without fully destroying the
  *       OBS context.
  *
  * @param   ovi  Pointer to an obs_video_info structure containing the
  *               specification of the graphics subsystem,
- * @return       OBS_VIDEO_SUCCESS if sucessful
+ * @return       OBS_VIDEO_SUCCESS if successful
  *               OBS_VIDEO_NOT_SUPPORTED if the adapter lacks capabilities
  *               OBS_VIDEO_INVALID_PARAM if a parameter is invalid
  *               OBS_VIDEO_CURRENTLY_ACTIVE if video is currently active
@@ -335,7 +335,7 @@ EXPORT int obs_open_module(obs_module_t **module, const char *path,
 
 /**
  * Initializes the module, which calls its obs_module_load export.  If the
- * module is alrady loaded, then this function does nothing and returns
+ * module is already loaded, then this function does nothing and returns
  * successful.
  */
 EXPORT bool obs_init_module(obs_module_t *module);
@@ -533,8 +533,8 @@ enum obs_base_effect {
 EXPORT gs_effect_t *obs_get_base_effect(enum obs_base_effect effect);
 
 /* DEPRECATED: gets texture_rect default effect */
-DEPRECATED_START EXPORT gs_effect_t *obs_get_default_rect_effect(void)
-	DEPRECATED_END;
+DEPRECATED
+EXPORT gs_effect_t *obs_get_default_rect_effect(void);
 
 /** Returns the primary obs signal handler */
 EXPORT signal_handler_t *obs_get_signal_handler(void);
@@ -582,6 +582,15 @@ EXPORT enum obs_obj_type obs_obj_get_type(void *obj);
 EXPORT const char *obs_obj_get_id(void *obj);
 EXPORT bool obs_obj_invalid(void *obj);
 
+typedef bool (*obs_enum_audio_device_cb)(void *data, const char *name,
+		const char *id);
+
+EXPORT void obs_enum_audio_monitoring_devices(obs_enum_audio_device_cb cb,
+		void *data);
+
+EXPORT bool obs_set_audio_monitoring_device(const char *name, const char *id);
+EXPORT void obs_get_audio_monitoring_device(const char **name, const char **id);
+
 
 /* ------------------------------------------------------------------------- */
 /* View context */
@@ -610,6 +619,8 @@ EXPORT void obs_view_render(obs_view_t *view);
 
 EXPORT uint64_t obs_get_video_frame_time(void);
 
+EXPORT double obs_get_active_fps(void);
+
 
 /* ------------------------------------------------------------------------- */
 /* Display context */
@@ -858,6 +869,8 @@ EXPORT void obs_source_enum_filters(obs_source_t *source,
 EXPORT obs_source_t *obs_source_get_filter_by_name(obs_source_t *source,
 		const char *name);
 
+EXPORT void obs_source_copy_filters(obs_source_t *dst, obs_source_t *src);
+
 EXPORT bool obs_source_enabled(const obs_source_t *source);
 EXPORT void obs_source_set_enabled(obs_source_t *source, bool enabled);
 
@@ -912,6 +925,17 @@ EXPORT void obs_source_set_deinterlace_field_order(obs_source_t *source,
 EXPORT enum obs_deinterlace_field_order obs_source_get_deinterlace_field_order(
 		const obs_source_t *source);
 
+enum obs_monitoring_type {
+	OBS_MONITORING_TYPE_NONE,
+	OBS_MONITORING_TYPE_MONITOR_ONLY,
+	OBS_MONITORING_TYPE_MONITOR_AND_OUTPUT
+};
+
+EXPORT void obs_source_set_monitoring_type(obs_source_t *source,
+		enum obs_monitoring_type type);
+EXPORT enum obs_monitoring_type obs_source_get_monitoring_type(
+		const obs_source_t *source);
+
 /* ------------------------------------------------------------------------- */
 /* Functions used by sources */
 
@@ -952,6 +976,13 @@ EXPORT void obs_source_draw(gs_texture_t *image, int x, int y,
 EXPORT void obs_source_output_video(obs_source_t *source,
 		const struct obs_source_frame *frame);
 
+/** Preloads asynchronous video data to allow instantaneous playback */
+EXPORT void obs_source_preload_video(obs_source_t *source,
+		const struct obs_source_frame *frame);
+
+/** Shows any preloaded video data */
+EXPORT void obs_source_show_preloaded_video(obs_source_t *source);
+
 /** Outputs audio data (always asynchronous) */
 EXPORT void obs_source_output_audio(obs_source_t *source,
 		const struct obs_source_audio *audio);
@@ -1140,7 +1171,7 @@ EXPORT void obs_transition_swap_end(obs_source_t *tr_dest,
  * Creates a scene.
  *
  *   A scene is a source which is a container of other sources with specific
- * display oriantations.  Scenes can also be used like any other source.
+ * display orientations.  Scenes can also be used like any other source.
  */
 EXPORT obs_scene_t *obs_scene_create(const char *name);
 
@@ -1202,7 +1233,7 @@ EXPORT obs_source_t *obs_sceneitem_get_source(const obs_sceneitem_t *item);
 EXPORT void obs_sceneitem_select(obs_sceneitem_t *item, bool select);
 EXPORT bool obs_sceneitem_selected(const obs_sceneitem_t *item);
 
-/* Functions for gettings/setting specific orientation of a scene item */
+/* Functions for getting/setting specific orientation of a scene item */
 EXPORT void obs_sceneitem_set_pos(obs_sceneitem_t *item, const struct vec2 *pos);
 EXPORT void obs_sceneitem_set_rot(obs_sceneitem_t *item, float rot_deg);
 EXPORT void obs_sceneitem_set_scale(obs_sceneitem_t *item,
@@ -1366,12 +1397,6 @@ EXPORT signal_handler_t *obs_output_get_signal_handler(
 /** Returns the procedure handler for an output */
 EXPORT proc_handler_t *obs_output_get_proc_handler(const obs_output_t *output);
 
-/**
- * Sets the current video media context associated with this output,
- * required for non-encoded outputs
- */
-EXPORT void obs_output_set_video(obs_output_t *output, video_t *video);
-
 /**
  * Sets the current audio/video media contexts associated with this output,
  * required for non-encoded outputs.  Can be null.
@@ -1458,6 +1483,13 @@ EXPORT uint32_t obs_output_get_height(const obs_output_t *output);
 
 EXPORT const char *obs_output_get_id(const obs_output_t *output);
 
+#if BUILD_CAPTIONS
+EXPORT void obs_output_output_caption_text1(obs_output_t *output,
+		const char *text);
+#endif
+
+EXPORT float obs_output_get_congestion(obs_output_t *output);
+
 /* ------------------------------------------------------------------------- */
 /* Functions used by outputs */
 
@@ -1649,11 +1681,17 @@ EXPORT const char *obs_encoder_get_id(const obs_encoder_t *encoder);
 EXPORT uint32_t obs_get_encoder_caps(const char *encoder_id);
 
 /** Duplicates an encoder packet */
+DEPRECATED
 EXPORT void obs_duplicate_encoder_packet(struct encoder_packet *dst,
 		const struct encoder_packet *src);
 
+DEPRECATED
 EXPORT void obs_free_encoder_packet(struct encoder_packet *packet);
 
+EXPORT void obs_encoder_packet_ref(struct encoder_packet *dst,
+		struct encoder_packet *src);
+EXPORT void obs_encoder_packet_release(struct encoder_packet *packet);
+
 
 /* ------------------------------------------------------------------------- */
 /* Stream Services */
