@@ -798,7 +798,7 @@ struct obs_source_info coreaudio_output_capture_info = {
 	.type           = OBS_SOURCE_TYPE_INPUT,
 	.output_flags   = OBS_SOURCE_AUDIO |
 	                  OBS_SOURCE_DO_NOT_DUPLICATE |
-	                  OBS_SOURCE_DO_NOT_MONITOR,
+	                  OBS_SOURCE_DO_NOT_SELF_MONITOR,
 	.get_name       = coreaudio_output_getname,
 	.create         = coreaudio_create_output_capture,
 	.destroy        = coreaudio_destroy,
