@@ -92,9 +92,6 @@ static void rtmp_stream_destroy(void *data)
 		}
 	}
 
-	if (stream->socket_thread_active)
-		pthread_join(stream->socket_thread, NULL);
-
 	free_packets(stream);
 	dstr_free(&stream->path);
 	dstr_free(&stream->key);
@@ -109,6 +106,13 @@ static void rtmp_stream_destroy(void *data)
 #ifdef TEST_FRAMEDROPS
 	circlebuf_free(&stream->droptest_info);
 #endif
+
+	os_event_destroy(stream->buffer_space_available_event);
+	os_event_destroy(stream->buffer_has_data_event);
+	os_event_destroy(stream->socket_available_event);
+	os_event_destroy(stream->send_thread_signaled_exit);
+	pthread_mutex_destroy(&stream->write_buf_mutex);
+
 	if (stream->write_buf)
 		bfree(stream->write_buf);
 	bfree(stream);
@@ -129,6 +133,32 @@ static void *rtmp_stream_create(obs_data_t *settings, obs_output_t *output)
 	if (os_event_init(&stream->stop_event, OS_EVENT_TYPE_MANUAL) != 0)
 		goto fail;
 
+	if (pthread_mutex_init(&stream->write_buf_mutex, NULL) != 0) {
+		warn("Failed to initialize write buffer mutex");
+		goto fail;
+	}
+
+	if (os_event_init(&stream->buffer_space_available_event,
+		OS_EVENT_TYPE_AUTO) != 0) {
+		warn("Failed to initialize write buffer event");
+		goto fail;
+	}
+	if (os_event_init(&stream->buffer_has_data_event,
+		OS_EVENT_TYPE_AUTO) != 0) {
+		warn("Failed to initialize data buffer event");
+		goto fail;
+	}
+	if (os_event_init(&stream->socket_available_event,
+		OS_EVENT_TYPE_AUTO) != 0) {
+		warn("Failed to initialize socket buffer event");
+		goto fail;
+	}
+	if (os_event_init(&stream->send_thread_signaled_exit,
+		OS_EVENT_TYPE_MANUAL) != 0) {
+		warn("Failed to initialize socket exit event");
+		goto fail;
+	}
+
 	UNUSED_PARAMETER(settings);
 	return stream;
 
@@ -390,6 +420,14 @@ static void *send_thread(void *data)
 		info("User stopped the stream");
 	}
 
+	if (stream->new_socket_loop) {
+		os_event_signal(stream->send_thread_signaled_exit);
+		os_event_signal(stream->buffer_has_data_event);
+		pthread_join(stream->socket_thread, NULL);
+		stream->socket_thread_active = false;
+		stream->rtmp.m_bCustomSend = false;
+	}
+
 	RTMP_Close(&stream->rtmp);
 
 	if (!stopping(stream)) {
@@ -540,26 +578,7 @@ static int init_send(struct rtmp_stream *stream)
 			return OBS_OUTPUT_ERROR;
 		}
 
-		if (pthread_mutex_init(&stream->write_buf_mutex, NULL) != 0) {
-			warn("Failed to initialize write buffer mutex");
-			return OBS_OUTPUT_ERROR;
-		}
-
-		if (os_event_init(&stream->buffer_space_available_event,
-					OS_EVENT_TYPE_MANUAL) != 0) {
-			warn("Failed to initialize write buffer event");
-			return OBS_OUTPUT_ERROR;
-		}
-		if (os_event_init(&stream->buffer_has_data_event,
-					OS_EVENT_TYPE_MANUAL) != 0) {
-			warn("Failed to initialize data buffer event");
-			return OBS_OUTPUT_ERROR;
-		}
-		if (os_event_init(&stream->socket_available_event,
-					OS_EVENT_TYPE_MANUAL) != 0) {
-			warn("Failed to initialize socket buffer event");
-			return OBS_OUTPUT_ERROR;
-		}
+		os_event_reset(stream->send_thread_signaled_exit);
 
 		info("New socket loop enabled by user");
 		if (stream->low_latency_mode)
@@ -568,8 +587,37 @@ static int init_send(struct rtmp_stream *stream)
 		if (stream->write_buf)
 			bfree(stream->write_buf);
 
-		stream->write_buf_size = STREAM_WRITE_BUFFER_SIZE;
-		stream->write_buf = bmalloc(STREAM_WRITE_BUFFER_SIZE);
+		int total_bitrate = 0;
+		obs_output_t  *context  = stream->output;
+
+		obs_encoder_t *vencoder = obs_output_get_video_encoder(context);
+		if (vencoder) {
+			obs_data_t *params = obs_encoder_get_settings(vencoder);
+			if (params) {
+				int bitrate = obs_data_get_int(params, "bitrate");
+				total_bitrate += bitrate;
+				obs_data_release(params);
+			}
+		}
+
+		obs_encoder_t *aencoder = obs_output_get_audio_encoder(context, 0);
+		if (aencoder) {
+			obs_data_t *params = obs_encoder_get_settings(aencoder);
+			if (params) {
+				int bitrate = obs_data_get_int(params, "bitrate");
+				total_bitrate += bitrate;
+				obs_data_release(params);
+			}
+		}
+
+		// to bytes/sec
+		int ideal_buffer_size = total_bitrate * 128;
+
+		if (ideal_buffer_size < 131072)
+			ideal_buffer_size = 131072;
+
+		stream->write_buf_size = ideal_buffer_size;
+		stream->write_buf = bmalloc(ideal_buffer_size);
 
 #ifdef _WIN32
 		ret = pthread_create(&stream->socket_thread, NULL,
@@ -670,7 +718,7 @@ static int try_connect(struct rtmp_stream *stream)
 
 	info("Connecting to RTMP URL %s...", stream->path.array);
 
-	memset(&stream->rtmp.Link, 0, sizeof(stream->rtmp.Link));
+	RTMP_Init(&stream->rtmp);
 	if (!RTMP_SetupURL(&stream->rtmp, stream->path.array))
 		return OBS_OUTPUT_BAD_PATH;
 
@@ -741,11 +789,6 @@ static bool init_connect(struct rtmp_stream *stream)
 		pthread_join(stream->send_thread, NULL);
 	}
 
-	if (stream->socket_thread_active) {
-		pthread_join(stream->socket_thread, NULL);
-		stream->socket_thread_active = false;
-	}
-
 	free_packets(stream);
 
 	service = obs_output_get_service(stream->output);
