@@ -1,5 +1,3 @@
-#define _CRT_SECURE_NO_WARNINGS
-
 #ifdef _MSC_VER
 #pragma warning(disable : 4214) /* nonstandard extension, non-int bitfield */
 #pragma warning(disable : 4054) /* function pointer to data pointer */
@@ -27,6 +25,8 @@ static struct func_hook wgl_swap_layer_buffers;
 static struct func_hook wgl_swap_buffers;
 static struct func_hook wgl_delete_context;
 
+static bool darkest_dungeon_fix = false;
+
 struct gl_data {
 	HDC                            hdc;
 	uint32_t                       base_cx;
@@ -37,6 +37,7 @@ struct gl_data {
 	GLuint                         fbo;
 	bool                           using_shtex : 1;
 	bool                           using_scale : 1;
+	bool                           shmem_fallback : 1;
 
 	union {
 		/* shared texture */
@@ -230,9 +231,14 @@ static void get_window_size(HDC hdc, uint32_t *cx, uint32_t *cy)
 	HWND hwnd = WindowFromDC(hdc);
         RECT rc = {0};
 
-        GetClientRect(hwnd, &rc);
-	*cx = rc.right;
-	*cy = rc.bottom;
+	if (darkest_dungeon_fix) {
+		*cx = 1920;
+		*cy = 1080;
+	} else {
+		GetClientRect(hwnd, &rc);
+		*cx = rc.right;
+		*cy = rc.bottom;
+	}
 }
 
 static inline bool gl_shtex_init_window(void)
@@ -520,21 +526,32 @@ static bool gl_shmem_init(HWND window)
 	return true;
 }
 
-static void gl_init(HDC hdc)
+#define INIT_SUCCESS         0
+#define INIT_FAILED         -1
+#define INIT_SHTEX_FAILED   -2
+
+static int gl_init(HDC hdc)
 {
 	HWND window = WindowFromDC(hdc);
+	int ret = INIT_FAILED;
 	bool success = false;
 	RECT rc = {0};
 
-	GetClientRect(window, &rc);
+	if (darkest_dungeon_fix) {
+		data.base_cx = 1920;
+		data.base_cy = 1080;
+	} else {
+		GetClientRect(window, &rc);
+		data.base_cx = rc.right;
+		data.base_cy = rc.bottom;
+	}
 
-	data.base_cx = rc.right;
-	data.base_cy = rc.bottom;
 	data.hdc = hdc;
 	data.format = DXGI_FORMAT_B8G8R8A8_UNORM;
 	data.using_scale = global_hook_info->use_scale;
 	data.using_shtex = nv_capture_available &&
-		!global_hook_info->force_shmem;
+		!global_hook_info->force_shmem &&
+		!data.shmem_fallback;
 
 	if (data.using_scale) {
 		data.cx = global_hook_info->cx;
@@ -544,13 +561,20 @@ static void gl_init(HDC hdc)
 		data.cy = data.base_cy;
 	}
 
-	if (data.using_shtex)
+	if (data.using_shtex) {
 		success = gl_shtex_init(window);
-	else
+		if (!success)
+			ret = INIT_SHTEX_FAILED;
+	} else {
 		success = gl_shmem_init(window);
+	}
 
 	if (!success)
 		gl_free();
+	else
+		ret = INIT_SUCCESS;
+
+	return ret;
 }
 
 static void gl_copy_backbuffer(GLuint dst)
@@ -572,9 +596,11 @@ static void gl_copy_backbuffer(GLuint dst)
 	}
 
 	glReadBuffer(GL_BACK);
-	if (gl_error("gl_copy_backbuffer", "failed to set read buffer")) {
-		return;
-	}
+
+	/* darkest dungeon fix */
+	darkest_dungeon_fix =
+		glGetError() == GL_INVALID_OPERATION &&
+		_strcmpi(process_name, "Darkest.exe") == 0;
 
 	glDrawBuffer(GL_COLOR_ATTACHMENT0);
 	if (gl_error("gl_copy_backbuffer", "failed to set draw buffer")) {
@@ -714,11 +740,17 @@ static void gl_capture(HDC hdc)
 		}
 	}
 
+	/* reset error flag */
+	glGetError();
+
 	if (capture_should_stop()) {
 		gl_free();
 	}
 	if (capture_should_init()) {
-		gl_init(hdc);
+		if (gl_init(hdc) == INIT_SHTEX_FAILED) {
+			data.shmem_fallback = true;
+			gl_init(hdc);
+		}
 	}
 	if (capture_ready() && hdc == data.hdc) {
 		uint32_t new_cx;
@@ -727,7 +759,8 @@ static void gl_capture(HDC hdc)
 		/* reset capture if resized */
 		get_window_size(hdc, &new_cx, &new_cy);
 		if (new_cx != data.base_cx || new_cy != data.base_cy) {
-			gl_free();
+			if (new_cx != 0 && new_cy != 0)
+				gl_free();
 			return;
 		}
 
